; SketchyLISP Library
; Copyright (C) 2005,2006,2007 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; >=

; ---conformance---
; R5RS

; ---purpose---
; Check whether a seqeunce of numbers is in strict
; non-ascending order.
; Return
; .C #t,
; if
; .V a>=b>=...
; and otherwise
; .C #f.

; ---args---
; A - number
; B... - numbers

; ---keywords---
; >= function, not less than, greater than or equal to,
; ordering, relation, comparison, predicate

; ---see-also---
; digits, <, >, <=, n>=

; ---example---
; (>= 4 4 3) => #t

(define gteq #t)

; (require "nullp.scm") ; null?
(require "less.scm") ; <
(require "fold-left.scm")
(require "neqp.scm") ; neq?

; ---code---
(define (>= a . b)
  (letrec
    ((gteq
       (lambda (a b)
         (cond ((eq? a #t) #t)
           ((< a b) #t)
           (else b)))))
    (cond ((null? b)
        (bottom '(too few arguments to >=)))
      (else (neq? (fold-left gteq a b) #t)))))
