; SketchyLISP Library
; Copyright (C) 2005,2006,2007 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; map

; ---conformance---
; R5RS

; ---purpose---
; Map a function over lists.
; The function
; .V f
; is applied to the list containing the
; .E car
; part of each argument, giving the first
; member of the result list.
; .C Map
; then proceeds with the
; .E cadr
; parts of each list, giving the next member of the
; result list, etc.
; .B
; All lists passed to
; .C map
; must have the same length.

; ---args---
; F - function to apply
; A* ... - lists

; ---keywords---
; MAP function, mapping, lists, CAR

; ---see-also---
; member, list?, fold-left, fold-right

; ---example---
; (map cons '(a b c) '(d e f)) => ((a . d) (b . e) (c . f))

(define map #t)

; (require "nullp.scm") ; null?
(require "reverse.scm")
(require "map-car.scm")

; ---code---
(define (map f . a*)
  (letrec
    ((carof
       (lambda (a)
         (map-car car a)))
     (cdrof
       (lambda (a)
         (map-car cdr a)))
     (_map
       (lambda (a* r)
         (cond ((null? (car a*)) (reverse r))
           (else (_map (cdrof a*)
                 (cons (apply f (carof a*)) r)))))))
    (cond ((null? a*)
        (bottom '(too few arguments to map)))
      (else (_map a* '())))))
