; SketchyLISP Library
; Copyright (C) 2005,2006,2007 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; -

; ---conformance---
; R5RS

; ---purpose---
; Subtract numbers. If only one argument is given,
; return its negative value.

; ---args---
; A - number
; B... - numbers (to be subtracted)

; ---keywords---
; - function, difference, subtract, subtraction
; arithmetics

; ---see-also---
; digits, +, *, quotient, remainder, n-

; ---example---
; (- 5 7) => -2

(define minus #t)

; (require "nullp.scm") ; null?
(require "digits.scm")
(require "abs.scm")
(require "plus.scm") ; +
(require "integer.scm")
(require "negate.scm")
(require "fold-left.scm")
(require "negativep.scm") ; negative?

; ---code---
(define (- a . b)
  (letrec
    ((idiff
       (lambda (a b)
         (cond ((negative? b)
             (+ a (abs b)))    ; B<0
           ((negative? a)
             (+ a (negate b))) ; B>=0, A<0
           ((n< (abs a) (abs b))
             (negate (n- (abs b) (abs a))))  ; A>=0, B>=0, A<B
           (else (n- (abs a) (abs b))))))    ; A>=0, B>=0, A>=B
     (i-
       (lambda (a b)
         (idiff (integer a) (integer b)))))
    (cond ((null? b) (negate a))
      (else (fold-left i- a b)))))
