; SketchyLISP Library
; Copyright (C) 2005,2006,2007 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; number->string

; ---conformance---
; R5RS
; Accepts only strings representing integers.

; ---purpose---
; Convert a number to a string. The radix must be in the
; range 2..16. If no radix is given, 10 is assumed.

; ---args---
; N - integer
; RADIX - optional radix

; ---keywords---
; NUMBER->STRING function, string, integer, number, conversion

; ---see-also---
; string->number

; ---example---
; (number->string -2748 16) => "-abc"

(define numtostr #t)

; (require "nullp.scm") ; null?
(require "zerop.scm") ; zero?
(require "times.scm") ; *
(require "quotient.scm")
(require "remainder.scm")
(require "less.scm") ; <
(require "negativep.scm") ; negative?
(require "abs.scm")

; ---code---
(define (number->string n . radix)
  (letrec
    ((digits
       (list->vector (string->list "0123456789abcdef")))
     (conv
       (lambda (n rdx res)
         (cond ((zero? n) res)
           (else (conv (quotient n rdx) rdx
                   (cons (vector-ref digits (remainder n rdx))
                         res))))))
     (get-radix
       (lambda ()
         (cond ((null? radix) 10)
           ((< 1 (car radix) 17) (car radix))
           (else (bottom '(bad radix in number->string)))))))
    (let ((r (get-radix)))
      (cond ((= r 10) (write-to-string (+ n 0)))
        ((zero? n) "0")
        ((negative? n)
          (list->string
            (cons #\- (conv (abs n) r '()))))
        (else (list->string (conv n r '())))))))
