; SketchyLISP Library
; Copyright (C) 2006,2007 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; predicate-iterator

; ---conformance---
; SketchyLISP Core

; ---purpose---
; Turn a binary predicate into a variadic predicate.

; ---args---
; P - predicate

; ---keywords---
; PREDICATE-ITERATOR function, predicate, iteration, binary, variadic

; ---see-also---
; <, =, char<?, char=?, string<?, string=?

; ---example---
; ((predicate-iterator eq?) 'foo 'foo 'foo) => #t

(define predicate-iterator #t)

(require "nullp.scm") ; null?
(require "neqp.scm") ; neq?
(require "fold-left.scm")

; ---code---
(define (predicate-iterator p)
  (lambda (first . rest)
    (let ((fail (cons 'fail '())))
      (let
        ((comp
           (lambda (a b)
             (cond ((eq? a fail) fail)
               ((p a b) b)
               (else fail)))))
        (cond ((null? rest)
            (bottom '(too few arguments)))
          (else (neq? (fold-left comp first rest)
                      fail)))))))
