; SketchyLISP Library
; Copyright (C) 2005,2006,2007 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; string-ci<?

; ---conformance---
; R5RS

; ---purpose---
; Case-insensitively test whether a sequence of strings
; is in lexically ascending order.

; ---args---
; X - string
; Y... - strings

; ---keywords---
; STRING-CI<? function, strings, comparison, lexical order,
; ascending, case-insensitive

; ---see-also---
; string<?, string-ci=?, string-ci>?, string-ci<=?, string-ci>=?, char-ci<?
; equal?

; ---example---
; (string-ci<? "abc" "XYZ" "zZz") => #t

(define s-ciltp #t)

; (require "c-cigtp.scm") ; char-ci>?
; (require "c-ciltp.scm") ; char-ci<?
; (load "nullp.scm") ; null?
(load "not.scm")
(require "pred-iter.scm") ; predicate-iterator

; ---code---
(define string-ci<?
  (letrec
    ((ci-lt?
       (lambda (x y)
         (cond ((null? x) (not (null? y)))
           ((null? y) #f)
           ((char-ci<? (car x) (car y)) #t)
           ((char-ci>? (car x) (car y)) #f)
           (else (ci-lt? (cdr x) (cdr y)))))))
  (predicate-iterator
    (lambda (x y)
      (ci-lt? (string->list x)
              (string->list y))))))
