; SketchyLISP Library
; Copyright (C) 2005,2006,2007 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; *

; ---conformance---
; R5RS

; ---purpose---
; Multiply numbers.

; ---args---
; A... - numbers

; ---keywords---
; * function, times, multiply, multiplication, product
; arithmetics

; ---see-also---
; digits, +, -, quotient, remainder, n*

; ---example---
; (* 2 3 4) => 24

(define times #t)

(require "digits.scm")
(require "abs.scm")
(require "minus.scm") ; -
(require "integer.scm")
(require "ntimes.scm") ; n*
(require "zerop.scm") ; zero?
(require "negate.scm")
(require "fold-left.scm")

; ---code---
(define (* . a)
  (letrec
    ((_itimes
       (lambda (a b)
         (cond
           ; avoid leading zeroes in result
           ((zero? a) 0)
           ; handle signs
           ((eq? (negative? a) (negative? b))
             (n* (abs a) (abs b)))
           (else (negate (n* (abs a) (abs b)))))))
     (i*
       (lambda (a b)
         (_itimes (integer a) (integer b)))))
    (fold-left i* 1 a)))
